@@ -144,7 +144,7 @@ class ContractMethod:
         else:
             raise TypeError("wrong number of arguments, require {}".format(len(self.inputs)))
 
-        print("debug params: {}".format(parameter))
+
 
         paramdict = dict(
             contract_address=self.contract_address,
@@ -156,11 +156,11 @@ class ContractMethod:
 
         if self._abi.get("stateMutability", None).lower() in ["view", "pure"]:
             # const call, contract ret
-            ret = self.transaction_builder.trigger_smart_contract(**paramdict)
+            ret = self.transaction_builder.trigger_smart_contract(paramdict)
             return self.handle_ret(ret)
         else:
             paramdict.setdefault('parameters', []).append(parameter)
-            ret = self.transaction_builder.trigger_smart_contract(**paramdict)
+            ret = self.transaction_builder.trigger_smart_contract(paramdict)
             return self.handle_ret(ret)
 
     @property
