@@ -476,6 +476,10 @@ class TransactionBuilder(object):
         # is an integer which should be greater than 0.
         origin_energy_limit = kwargs.setdefault('origin_energy_limit', 10000000)
 
+        # parameters = kwargs.setdefault('parameters', []).append()
+        # parameters = kwargs.setdefault('visible', []).append()
+        # parameters = kwargs.setdefault('permission_id', []).append()
+
         if not is_integer(user_fee_percentage) and not user_fee_percentage:
             user_fee_percentage = 100
 
@@ -505,37 +509,16 @@ class TransactionBuilder(object):
 
         return self.tron.manager.request('/wallet/deploycontract', transaction)
 
-    def trigger_smart_contract(self, **kwargs):
-        print(kwargs)
-        """Trigger Smart Contract
-        Calls a function on a contract
+    def trigger_smart_contract_constant(self, kv: dict):
 
-        Args:
-            **kwargs: Fill in the required parameters
+        contract_address = kv.setdefault('contract_address', None)
+        function_selector = kv.setdefault('function_selector', None)
 
-        Examples:
-            >>> tron = Tron()
-            >>> tron.transaction_builder.trigger_smart_contract(
-            >>>     contract_address='413c8143e98b3e2fe1b1a8fb82b34557505a752390',
-            >>>     function_selector='set(uint256,uint256)',
-            >>>     fee_limit=30000,
-            >>>     call_value=0,
-            >>>     parameters=[
-            >>>        {'type': 'int256', 'value': 1},
-            >>>        {'type': 'int256', 'value': 1}])
-
-        Returns:
-            TransactionExtention, TransactionExtention contains unsigned Transaction
-        """
-
-        contract_address = kwargs.setdefault('contract_address', None)
-        function_selector = kwargs.setdefault('function_selector', None)
-        parameters = kwargs.setdefault('parameters', [])
-        issuer_address = kwargs.setdefault('issuer_address', self.tron.default_address.hex)
-        call_value = kwargs.setdefault('call_value', 0)
-        fee_limit = kwargs.setdefault('fee_limit', 1000000000)
-        token_value = kwargs.setdefault('token_value', 0)
-        token_id = kwargs.setdefault('token_id', 0)
+        issuer_address = kv.setdefault('issuer_address', self.tron.default_address.hex)
+        call_value = kv.setdefault('call_value', 0)
+        fee_limit = kv.setdefault('fee_limit', 1000000000)
+        token_value = kv.setdefault('token_value', 0)
+        token_id = kv.setdefault('token_id', 0)
 
         if not is_integer(token_value) or token_value < 0:
             raise ValueError('Invalid options.tokenValue provided')
@@ -558,9 +541,10 @@ class TransactionBuilder(object):
         """remove all the spaces"""
         function_selector = function_selector.replace('/\s*/g', '')
 
-        if len(parameters) > 0:
+        if hasattr(kv, "parameters") and len(kv['parameters']) > 0:
             types = []
             values = []
+            parameters = kv['parameters']
             for abi in parameters:
                 if 'type' not in abi or not is_string(abi['type']):
                     raise ValueError('Invalid parameter type provided: ' + abi['type'])
@@ -577,11 +561,14 @@ class TransactionBuilder(object):
                 print(ex)
 
         else:
-            parameters = ''
+            parameters = []
+
+        print("==checking")
+        print(kv)
 
         data = {
-            'contract_address': self.tron.address.to_hex(contract_address),
-            'owner_address': self.tron.address.to_hex(issuer_address),
+            'contract_address': self.tron.toHex(text=contract_address),
+            'owner_address': issuer_address.hex,
             'function_selector': function_selector,
             'fee_limit': int(fee_limit),
             'call_value': int(call_value),
@@ -594,6 +581,107 @@ class TransactionBuilder(object):
         if token_id:
             data['token_id'] = int(token_id)
 
+        print("==data load")
+        print(data)
+
+        return self.tron.manager.request('/wallet/triggerconstantcontract', kv)
+
+    def trigger_smart_contract(self, kv: dict):
+
+        """Trigger Smart Contract
+        Calls a function on a contract
+
+        Args:
+            **kv: Fill in the required parameters
+
+        Examples:
+            >>> tron = Tron()
+            >>> tron.transaction_builder.trigger_smart_contract(
+            >>>     contract_address='413c8143e98b3e2fe1b1a8fb82b34557505a752390',
+            >>>     function_selector='set(uint256,uint256)',
+            >>>     fee_limit=30000,
+            >>>     call_value=0,
+            >>>     parameters=[
+            >>>        {'type': 'int256', 'value': 1},
+            >>>        {'type': 'int256', 'value': 1}])
+
+        Returns:
+            TransactionExtention, TransactionExtention contains unsigned Transaction
+        """
+
+        contract_address = kv.setdefault('contract_address', None)
+        function_selector = kv.setdefault('function_selector', None)
+
+        issuer_address = kv.setdefault('issuer_address', self.tron.default_address.hex)
+        call_value = kv.setdefault('call_value', 0)
+        fee_limit = kv.setdefault('fee_limit', 1000000000)
+        token_value = kv.setdefault('token_value', 0)
+        token_id = kv.setdefault('token_id', 0)
+
+        if not is_integer(token_value) or token_value < 0:
+            raise ValueError('Invalid options.tokenValue provided')
+
+        if not is_integer(token_id) or token_id < 0:
+            raise ValueError('Invalid options.tokenId provided')
+
+        if not self.tron.isAddress(contract_address):
+            raise InvalidAddress('Invalid contract address provided')
+
+        if not is_string(function_selector):
+            raise ValueError('Invalid function selector provided')
+
+        if not is_integer(call_value) or call_value < 0:
+            raise ValueError('Invalid call value provided')
+
+        if not is_integer(fee_limit) or fee_limit <= 0 or fee_limit > 1000000000:
+            raise ValueError('Invalid fee limit provided')
+
+        """remove all the spaces"""
+        function_selector = function_selector.replace('/\s*/g', '')
+
+        if hasattr(kv, "parameters") and len(kv['parameters']) > 0:
+            types = []
+            values = []
+            parameters = kv['parameters']
+            for abi in parameters:
+                if 'type' not in abi or not is_string(abi['type']):
+                    raise ValueError('Invalid parameter type provided: ' + abi['type'])
+
+                if abi['type'] == 'address':
+                    abi['value'] = self.tron.address.to_hex(abi['value']).replace('41', '0x', 1)
+
+                types.append(abi['type'])
+                values.append(abi['value'])
+
+            try:
+                parameters = encode_hex(encode_abi(types, values)).replace('0x', '', 2)
+            except ValueError as ex:
+                print(ex)
+
+        else:
+            parameters = []
+
+        print("==checking")
+        print(kv)
+
+        data = {
+            'contract_address': self.tron.toHex(text=contract_address),
+            'owner_address': issuer_address.hex,
+            'function_selector': function_selector,
+            'fee_limit': int(fee_limit),
+            'call_value': int(call_value),
+            'parameter': parameters
+        }
+
+        if token_value:
+            data['call_token_value'] = int(token_value)
+
+        if token_id:
+            data['token_id'] = int(token_id)
+
+        print("==data load")
+        print(data)
+
         return self.tron.manager.request('/wallet/triggersmartcontract', data)
 
     def create_trx_exchange(self,
