@@ -1,3 +1,5 @@
+#!/usr/bin/env python
+#
 # --------------------------------------------------------------------
 # Copyright (c) iEXBase. All rights reserved.
 # Licensed under the MIT License.
@@ -20,15 +22,16 @@ from typing import Any
 from trx_utils import is_integer, is_hex
 from trx_utils.types import is_object, is_string, is_list
 
-from tronpytool.common.transactions import wait_for_transaction_id
-from tronpytool.contract import Contract
-from tronpytool.exceptions import InvalidTronError, TronError, TimeExhausted
-from tronpytool.module import Module
+from tronpytool.common import key
+from tronpytool.common.account import Account
 from tronpytool.common.blocks import select_method_for_block
 from tronpytool.common.toolz import (
     assoc
 )
-from tronpytool.common.account import Account
+from tronpytool.common.transactions import wait_for_transaction_id
+from tronpytool.contract import Contract
+from tronpytool.exceptions import InvalidTronError, TronError, TimeExhausted, AssetNotFound
+from tronpytool.module import Module
 
 TRX_MESSAGE_HEADER = '\x19TRON Signed Message:\n'
 ETH_MESSAGE_HEADER = '\x19Ethereum Signed Message:\n'
@@ -539,7 +542,8 @@ class Trx(Module):
             'privateKey': self.tron.private_key
         })
 
-    def sign(self, transaction: Any, use_tron: bool = True, multisig: bool = False):
+    def sign(self, transaction: Any, private_key=None, use_tron: bool = True, multisig: bool = False,
+             is_message_hex=False):
         """Safe method for signing your transaction
 
         Warnings:
@@ -561,17 +565,24 @@ class Trx(Module):
             header = TRX_MESSAGE_HEADER if use_tron else ETH_MESSAGE_HEADER
             header += str(len(transaction))
 
-            message_hash = self.tron.keccak(text=header+transaction)
+            if is_message_hex:
+                from eth_hash.auto import keccak as keccak_256
+                message_hash = keccak_256(header.encode('utf-8') + bytes.fromhex(transaction))
+            else:
+                message_hash = self.tron.keccak(text=header + transaction)
 
-            signed_message = Account.sign_hash(self.tron.toHex(message_hash), self.tron.private_key)
+            signed_message = Account.sign_hash(self.tron.toHex(message_hash), private_key or self.tron.private_key)
             return signed_message
 
         if not multisig and 'signature' in transaction:
             raise TronError('Transaction is already signed')
 
         try:
+            if transaction.get('transaction', ''):
+                transaction = transaction['transaction']
+
             if not multisig:
-                address = self.tron.address.from_private_key(self.tron.private_key).hex.lower()
+                address = self.tron.address.from_private_key(private_key or self.tron.private_key).hex.lower()
                 owner_address = transaction['raw_data']['contract'][0]['parameter']['value']['owner_address']
 
                 if address != owner_address:
@@ -579,7 +590,7 @@ class Trx(Module):
 
             # This option deals with signing of transactions, and writing to the array
             signed_tx = Account.sign_hash(
-                transaction['txID'], self.tron.private_key
+                transaction['txID'], private_key or self.tron.private_key
             )
             signature = signed_tx['signature'].hex()[2:]
 
@@ -628,7 +639,7 @@ class Trx(Module):
         signed_tx = self.sign(transaction)
         return self.broadcast(signed_tx)
 
-    def verify_message(self, message, signed_message=None, address=None, use_tron: bool = True):
+    def verify_message(self, message, signed_message=None, address=None, use_tron: bool = True, is_message_hex=False):
         """ Get the address of the account that signed the message with the given hash.
         You must specify exactly one of: vrs or signature
 
@@ -650,7 +661,11 @@ class Trx(Module):
         header = TRX_MESSAGE_HEADER if use_tron else ETH_MESSAGE_HEADER
         header += str(len(message))
 
-        message_hash = self.tron.keccak(text=header+message)
+        if is_message_hex:
+            from eth_hash.auto import keccak as keccak_256
+            message_hash = keccak_256(header.encode('utf-8') + bytes.fromhex(message))
+        else:
+            message_hash = self.tron.keccak(text=header + message)
         recovered = Account.recover_hash(self.tron.toHex(message_hash), signed_message.signature)
 
         tron_address = '41' + recovered[2:]
@@ -816,6 +831,30 @@ class Trx(Module):
             'offset': int(offset)
         })
 
+    def get_asset_from_name(self, name: str) -> dict:
+        """Get asset info from its abbr name, might fail if there're duplicates."""
+        assets = [asset for asset in self.list_assets(1000, 0) if asset['abbr'] == name]
+        if assets:
+            if len(assets) == 1:
+                return assets[0]
+            raise ValueError("duplicated assets with the same name", [asset['id'] for asset in assets])
+        raise AssetNotFound
+
+    def list_assets(self, limit=0, offset=0) -> list:
+        """List all TRC10 tokens(assets)."""
+        assets = self.list_tokens(limit, offset)
+        for asset in assets:
+            asset["id"] = int(asset["id"])
+            asset["owner_address"] = key.to_base58check_address(asset["owner_address"])
+            asset["name"] = bytes.fromhex(asset["name"]).decode()
+            if "abbr" in asset:
+                asset["abbr"] = bytes.fromhex(asset["abbr"]).decode()
+            else:
+                asset["abbr"] = ""
+            asset["description"] = bytes.fromhex(asset["description"]).decode("utf8", "replace")
+            asset["url"] = bytes.fromhex(asset["url"]).decode()
+        return assets
+
     def time_until_next_vote_cycle(self):
         """Get the time of the next Super Representative vote
 
@@ -856,6 +895,7 @@ class Trx(Module):
             **kwargs (any): details (bytecode, abi)
         """
         factory_class = kwargs.pop('contract_factory_class', self.default_contract_factory)
+
         contract_factory = factory_class.factory(self.tron, **kwargs)
 
         if address:
